==> org-lsst-ccs-subsystem-fcs-main-8.1.1-sources.jar: org/lsst/ccs/subsystems/fcs/simulation/SimuCarouselController.java <==

package org.lsst.ccs.subsystems.fcs.simulation;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.commons.annotations.LookupField;
import static org.lsst.ccs.commons.annotations.LookupField.Strategy.TREE;
import org.lsst.ccs.subsystems.fcs.Carousel;
import org.lsst.ccs.subsystems.fcs.EPOSEnumerations;
import org.lsst.ccs.subsystems.fcs.EPOSEnumerations.EposMode;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.EposMode.PROFILE_POSITION;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.EposState.SWITCH_ON_DISABLED;
import org.lsst.ccs.subsystems.fcs.EPOSEnumerations.Parameter;

import org.lsst.ccs.subsystems.fcs.drivers.CanOpenEPOSCarousel;
import org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException;

/**
 * To simulate the controller which rotates carousel.
 * @author virieux
 */
public class SimuCarouselController extends CanOpenEPOSCarousel {
    private static final Logger FCSLOG = Logger.getLogger(SimuCarouselController.class.getName());

    @LookupField(strategy = TREE)
    private Carousel carousel;

    /*CANOpen devices to read the values of the clamps sensors.*/
    @LookupField(strategy = TREE)
    private SimuTTC580 hyttc580;

    protected int targetPosition;
    protected int ssiPosition;
    protected int targetCurrent;
    protected int actualCurrent;

    /**
     * Build a new SimuCarouselController
     */
    public SimuCarouselController() {
        this.position = 0;
        this.mode = PROFILE_POSITION;
        this.simuEnabled = false;
        this.statusWord = 0x40;
        this.eposState = SWITCH_ON_DISABLED;
    }

    @Override
    public void init() {
        super.init();
        updateFakePDOData(cobid1, 0);
        updateFakePDOData(cobid2, 0);
    }

    @Override
    public void build() {
        super.build();
        this.statusWord = 0x40;
        simulateStatusWord(statusWord);
        this.setProfileAcceleration(2000);
        this.setProfileDeceleration(1000);
        this.setProfileVelocity(3400);
    }

    @Override
    public void goToSwitchOnDisabled() {
        simulateSWITCH_ON_DISABLED();
    }

    /**
     * To initialize a simulated ssiPosition.
     * @param ssiPosition
     */
    public void setSsiPosition(int ssiPosition) {
        this.ssiPosition = ssiPosition;
        this.position = ssiPosition;
        simulatePosition(ssiPosition);
    }

    @Override
    public void setPositionSensorTypeSinusIncrementalEncoder() {
        setPositionSensorType(8);
    }

    @Override
    public void setPositionSensorTypeEncoderSSI() {
        setPositionSensorType(4);
    }

    /**
     * This methods does nothing but save the relativePosition given as argument.
     * @param absolutePosition
     * @throws org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING_ROUTINE,
            description = "In PROFILE_POSITION mode this methods set the target position.")
    @Override
    public void writeTargetPosition(int absolutePosition) throws FcsHardwareException {
        FCSLOG.info(name + " => carousel is rotating.");
        FCSLOG.info(name + " => initial position=" + this.position);
        FCSLOG.info(name + " => target position=" + absolutePosition);
        int iterNB = 10;
        int initialPosition = carousel.getPosition();
        int step = (absolutePosition - initialPosition) / iterNB;
        FCSLOG.info("step=" + step);
        try {
            for (int i = 1; i < iterNB; i++) {
                int newPos = initialPosition + (i * step);
                setSsiPosition(newPos);
                FCSLOG.info(name + " i=" + i + ", position=" + position);
                try {
                    Thread.sleep(50);
                    if (carousel.getHaltRequired().get()) {
                        FCSLOG.fine(() -> name + " STOP simulated carousel motion.");
                        return;
                    }
                } catch (InterruptedException ex) {
                    throw new FcsHardwareException(name + " sleep was interrupted.",ex);
                }
            }
            int finalPos = absolutePosition % carousel.getFullTurn();
            setSsiPosition(finalPos);
            hyttc580.simulateSocketSensors(finalPos);
            carousel.updateStateWithSensors();

        } catch (FcsHardwareException  ex) {
            FCSLOG.log(Level.SEVERE, name+" should not raise an Exception:",ex);
        }
    }

    /**
     * Update PDOdata
     * @param cobid
     * @param newPdo
     */
    public void updateFakePDOData(int cobid, long newPdo) {
        if (cobid == cobid1) {
            setPdo1(newPdo);
        } else if (cobid == cobid2) {
            setPdo2(newPdo);
        }
        ((SimuCanOpenInterface) tcpProxy.getCanInterface()).simulatePDOData(cobid, newPdo);
    }

    /**
     * set controller position in PDOData to simulate a carousel position.
     * @param pos
     */
    public void simulatePosition(int pos) {
        FCSLOG.finer("simulate carousel position : " + pos);
        long pdo = (long) pos << 32;
        long mask = 0x00000000FFFFFFFFL;
        long newpdo = pdo | (getPdo1() & mask);
        updateFakePDOData(cobid1, newpdo);
    }

    /**
     * set carouselController statusWord in PDOData to simulate a statusWord.
     * @param sw
     */
    public void simulateStatusWord(int sw) {
        FCSLOG.finer("simulate carouselController statusWord : " + sw);
        long pdo = (long) sw;
        long mask = 0xFFFFFFFFFFFF0000L;
        long newpdo = pdo | (getPdo1() & mask);
        updateFakePDOData(cobid1, newpdo);
    }

    public void simulateSWITCH_ON_DISABLED() {
        simulateStatusWord(0x40);
        carousel.updateStateWithSensors();
    }

    /**
     * **************************************************************************
     *
     * Methods copied from SimuEPOSController. This is ugly to copy methods.
     * //TODO see how we can extract an interface with this methods.
     *
     * **************************************************************************
     */
    /**
     * set a position for simulation
     *
     * @param actualPosition
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING_ROUTINE,
            description = "For simulator only : Update position with a position given as argument.")
    public void setPosition(int actualPosition) {
        this.position = actualPosition;
    }

    /**
     * overriden for simulation
     *
     * @return
     */
    @Override
    public boolean isTargetReached() {
        return this.position == this.targetPosition
                || this.actualCurrent == this.targetCurrent;
    }

    /**
     * We assume that simulated hardware'subs controller has always correct
     * parameters.
     *
     * @return
     */
    @Override
    public boolean isParametersOK() {
        return true;
    }



    @Override
    public void defineAbsolutePosition(int position) {
        this.position = position;
    }

    @Override
    public void writeParameters(EposMode mode) {
    }

    @Override
    public void writeParameter(Parameter param, int value) {
        if (param == Parameter.PositionSensorType) {
            setPositionSensorType(value);
        }
    }

    @Override
    public long readParameter(EPOSEnumerations.Parameter parameter) {
        if (parameter == Parameter.PositionSensorType) {
            return positionSensorType;
        } else {
            return 0;
        }
    }

    @Override
    public void writeParameters(Map<String, Integer> paramMap) {
    }

    @Override
    public void writeCurrent(int aValue) {
        this.actualCurrent = aValue;
    }

    @Override
    public void stopAction() {
    }

    @Override
    public int readPosition() {
        return this.position;
    }

    /**
     * Read the position returned by the absolute encoder (single serial data).
     *
     * @return
     */
    @Override
    public int readSSIPosition() {
        return this.ssiPosition;
    }

    @Override
    public int readCurrent() {
        return actualCurrent;
    }

    @Override
    public EPOSEnumerations.EposMode readMode() {
        return mode;
    }

    @Override
    public int readStatusWord() {
        return statusWord;
    }

    @Override
    public void quickStop() {
        FCSLOG.finest(() -> name + " quickStop done.");
    }

    @Override
    public int getErrorHistoryNB() {
        return 0;
    }

    @Override
    public int readErrorRegister() {
        return 0;
    }

    @Override
    public void updateErrorHistory() {
        errorHistory = new int[0];
    }

    @Override
    public String updateAndDisplayErrorHistory() {
        this.updateErrorHistory();
        return "No ERROR";
    }

    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING_ROUTINE,
            description = "Check if the Controller is in fault.")
    @Override
    public void checkFault() {
    }

    @Override
    public void checkFault(boolean withPublication) {
    }

    @Override
    public String readParametersForMode(EPOSEnumerations.EposMode mode) {
        return getName() + ":no parameter to read.";
    }

    @Override
    public void checkParameters(EPOSEnumerations.EposMode aMode) {
        /*nothing to do because no controller CPU*/
    }

    @Override
    public void writeControlWord(int w) {
        FCSLOG.finer(name + " writeControlWord : 0x" + Integer.toHexString(w) + " (in decimal:" + w + ")");
        switch (w) {
            case 7:
                //disableOperation
                this.statusWord = 0x23;
                break;
            case 0:
                //disableVoltage
                this.statusWord = 0x40;
                break;
            case 6:
                //shutdownController
                this.statusWord = 0x21;
                break;
            case 0xF:
                //switchOnEnableOperation
                this.statusWord = 0x27;
                break;
            default:
                this.statusWord = w;
                break;
        }
        simulateStatusWord(statusWord);
        FCSLOG.info(name + " statusWord = 0x" + Integer.toHexString(statusWord));
    }
}

==> org-lsst-ccs-subsystem-fcs-main-9.0.2-sources.jar: org/lsst/ccs/subsystems/fcs/simulation/SimuCarouselController.java <==

package org.lsst.ccs.subsystems.fcs.simulation;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.lsst.ccs.command.annotations.Command;
import org.lsst.ccs.commons.annotations.LookupField;
import static org.lsst.ccs.commons.annotations.LookupField.Strategy.TREE;
import org.lsst.ccs.subsystems.fcs.Carousel;
import org.lsst.ccs.subsystems.fcs.EPOSEnumerations;
import org.lsst.ccs.subsystems.fcs.EPOSEnumerations.EposMode;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.EposMode.PROFILE_POSITION;
import static org.lsst.ccs.subsystems.fcs.EPOSEnumerations.EposState.SWITCH_ON_DISABLED;
import org.lsst.ccs.subsystems.fcs.EPOSEnumerations.EposParameter;

import org.lsst.ccs.subsystems.fcs.drivers.CanOpenEPOSCarousel;
import org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException;

/**
 * To simulate the controller which rotates carousel.
 * @author virieux
 */
public class SimuCarouselController extends CanOpenEPOSCarousel {
    private static final Logger FCSLOG = Logger.getLogger(SimuCarouselController.class.getName());

    @LookupField(strategy = TREE)
    private Carousel carousel;

    /*CANOpen devices to read the values of the clamps sensors.*/
    @LookupField(strategy = TREE)
    private SimuTTC580 hyttc580;

    protected int targetPosition;
    protected int ssiPosition;
    protected int targetCurrent;
    protected int actualCurrent;

    /**
     * Build a new SimuCarouselController
     */
    public SimuCarouselController() {
        this.position = 0;
        this.mode = PROFILE_POSITION;
        this.simuEnabled = false;
        this.statusWord = 0x40;
        this.eposState = SWITCH_ON_DISABLED;
    }

    @Override
    public void init() {
        super.init();
        updateFakePDOData(cobid1, 0);
        updateFakePDOData(cobid2, 0);
    }

    @Override
    public void build() {
        super.build();
        this.statusWord = 0x40;
        simulateStatusWord(statusWord);
        this.setProfileAcceleration(2000);
        this.setProfileDeceleration(1000);
        this.setProfileVelocity(3400);
    }

    @Override
    public void goToSwitchOnDisabled() {
        simulateSWITCH_ON_DISABLED();
    }

    /**
     * To initialize a simulated ssiPosition.
     * @param ssiPosition
     */
    public void setSsiPosition(int ssiPosition) {
        this.ssiPosition = ssiPosition;
        this.position = ssiPosition;
        simulatePosition(ssiPosition);
    }

    @Override
    public void setPositionSensorTypeSinusIncrementalEncoder() {
        setPositionSensorType(8);
    }

    @Override
    public void setPositionSensorTypeEncoderSSI() {
        setPositionSensorType(4);
    }

    /**
     * This methods does nothing but save the relativePosition given as argument.
     * @param absolutePosition
     * @throws org.lsst.ccs.subsystems.fcs.errors.FcsHardwareException
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING_ROUTINE,
            description = "In PROFILE_POSITION mode this methods set the target position.")
    @Override
    public void writeTargetPosition(int absolutePosition) throws FcsHardwareException {
        FCSLOG.info(name + " => carousel is rotating.");
        FCSLOG.info(name + " => initial position=" + this.position);
        FCSLOG.info(name + " => target position=" + absolutePosition);
        int iterNB = 10;
        int initialPosition = carousel.getPosition();
        int step = (absolutePosition - initialPosition) / iterNB;
        FCSLOG.info("step=" + step);
        try {
            for (int i = 1; i < iterNB; i++) {
                int newPos = initialPosition + (i * step);
                setSsiPosition(newPos);
                FCSLOG.info(name + " i=" + i + ", position=" + position);
                try {
                    Thread.sleep(50);
                    if (carousel.getHaltRequired().get()) {
                        FCSLOG.fine(() -> name + " STOP simulated carousel motion.");
                        return;
                    }
                } catch (InterruptedException ex) {
                    throw new FcsHardwareException(name + " sleep was interrupted.",ex);
                }
            }
            int finalPos = absolutePosition % carousel.getFullTurn();
            setSsiPosition(finalPos);
            hyttc580.simulateSocketSensors(finalPos);
            carousel.updateStateWithSensors();

        } catch (FcsHardwareException  ex) {
            FCSLOG.log(Level.SEVERE, name+" should not raise an Exception:",ex);
        }
    }

    /**
     * Update PDOdata
     * @param cobid
     * @param newPdo
     */
    public void updateFakePDOData(int cobid, long newPdo) {
        if (cobid == cobid1) {
            setPdo1(newPdo);
        } else if (cobid == cobid2) {
            setPdo2(newPdo);
        }
        ((SimuCanOpenInterface) tcpProxy.getCanInterface()).simulatePDOData(cobid, newPdo);
    }

    /**
     * set controller position in PDOData to simulate a carousel position.
     * @param pos
     */
    public void simulatePosition(int pos) {
        FCSLOG.finer("simulate carousel position : " + pos);
        long pdo = (long) pos << 32;
        long mask = 0x00000000FFFFFFFFL;
        long newpdo = pdo | (getPdo1() & mask);
        updateFakePDOData(cobid1, newpdo);
    }

    /**
     * set carouselController statusWord in PDOData to simulate a statusWord.
     * @param sw
     */
    public void simulateStatusWord(int sw) {
        FCSLOG.finer("simulate carouselController statusWord : " + sw);
        long pdo = (long) sw;
        long mask = 0xFFFFFFFFFFFF0000L;
        long newpdo = pdo | (getPdo1() & mask);
        updateFakePDOData(cobid1, newpdo);
    }

    public void simulateSWITCH_ON_DISABLED() {
        simulateStatusWord(0x40);
        carousel.updateStateWithSensors();
    }

    /**
     * **************************************************************************
     *
     * Methods copied from SimuEPOSController. This is ugly to copy methods.
     * //TODO see how we can extract an interface with this methods.
     *
     * **************************************************************************
     */
    /**
     * set a position for simulation
     *
     * @param actualPosition
     */
    @Command(type = Command.CommandType.ACTION, level = Command.ENGINEERING_ROUTINE,
            description = "For simulator only : Update position with a position given as argument.")
    public void setPosition(int actualPosition) {
        this.position = actualPosition;
    }

    /**
     * overriden for simulation
     *
     * @return
     */
    @Override
    public boolean isTargetReached() {
        return this.position == this.targetPosition
                || this.actualCurrent == this.targetCurrent;
    }

    /**
     * We assume that simulated hardware'subs controller has always correct
     * parameters.
     *
     * @return
     */
    @Override
    public boolean isParametersOK() {
        return true;
    }



    @Override
    public void defineAbsolutePosition(int position) {
        this.position = position;
    }

    @Override
    public void writeParameters(EposMode mode) {
    }

    @Override
    public void writeParameter(EposParameter param, int value) {
        if (param == EposParameter.PositionSensorType) {
            setPositionSensorType(value);
        }
    }

    @Override
    public long readParameter(EPOSEnumerations.EposParameter parameter) {
        if (parameter == EposParameter.PositionSensorType) {
            return positionSensorType;
        } else {
            return 0;
        }
    }

    @Override
    public void writeParameters(Map<String, Integer> paramMap) {
    }

    @Override
    public void writeCurrent(int aValue) {
        this.actualCurrent = aValue;
    }

    @Override
    public void stopAction() {
    }

    @Override
    public int readPosition() {
        return this.position;
    }

    /**
     * Read the position returned by the absolute encoder (single serial data).
     *
     * @return
     */
    @Override
    public int readSSIPosition() {
        return this.ssiPosition;
    }

    @Override
    public int readCurrent() {
        return actualCurrent;
    }

    @Override
    public EPOSEnumerations.EposMode readMode() {
        return mode;
    }

    @Override
    public int readStatusWord() {
        return statusWord;
    }

    @Override
    public void quickStop() {
        FCSLOG.finest(() -> name + " quickStop done.");
    }

    @Override
    public int getErrorHistoryNB() {
        return 0;
    }

    @Override
    public int readErrorRegister() {
        return 0;
    }

    @Override
    public void updateErrorHistory() {
        errorHistory = new int[0];
    }

    @Override
    public String updateAndDisplayErrorHistory() {
        this.updateErrorHistory();
        return "No ERROR";
    }

    @Command(type = Command.CommandType.QUERY, level = Command.ENGINEERING_ROUTINE,
            description = "Check if the Controller is in fault.")
    @Override
    public void checkFault() {
    }

    @Override
    public void checkFault(boolean withPublication) {
    }

    @Override
    public String readParametersForMode(EPOSEnumerations.EposMode mode) {
        return getName() + ":no parameter to read.";
    }

    @Override
    public void checkParameters(EPOSEnumerations.EposMode aMode) {
        /*nothing to do because no controller CPU*/
    }

    @Override
    public void writeControlWord(int w) {
        FCSLOG.finer(name + " writeControlWord : 0x" + Integer.toHexString(w) + " (in decimal:" + w + ")");
        switch (w) {
            case 7:
                //disableOperation
                this.statusWord = 0x23;
                break;
            case 0:
                //disableVoltage
                this.statusWord = 0x40;
                break;
            case 6:
                //shutdownController
                this.statusWord = 0x21;
                break;
            case 0xF:
                //switchOnEnableOperation
                this.statusWord = 0x27;
                break;
            default:
                this.statusWord = w;
                break;
        }
        simulateStatusWord(statusWord);
        FCSLOG.info(name + " statusWord = 0x" + Integer.toHexString(statusWord));
    }
}
